@@ -53,7 +53,17 @@ sub _validate_isa ( $self, @ ) {
             my $possible_val = [];
 
             for ( $self->isa->@* ) {
-                push $possible_val->@*, $_ if index( $_, $val_ref->$*, 0 ) == 0;
+                if ( index( $_, $val_ref->$*, 0 ) == 0 ) {
+                    if ( length == length $val_ref->$* ) {
+
+                        # select current value if matched completely
+                        $possible_val = [$_];
+
+                        last;
+                    }
+
+                    push $possible_val->@*, $_;
+                }
             }
 
             if ( !$possible_val->@* ) {
@@ -75,14 +85,14 @@ sub _validate_isa ( $self, @ ) {
 ## -----SOURCE FILTER LOG BEGIN-----
 ##
 ## PerlCritic profile "pcore-script" policy violations:
-## ┌──────┬──────────────────────┬────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
-## │ Sev. │ Lines                │ Policy                                                                                                         │
-## ╞══════╪══════════════════════╪════════════════════════════════════════════════════════════════════════════════════════════════════════════════╡
-## │    3 │ 35                   │ Subroutines::ProhibitUnusedPrivateSubroutines - Private subroutine/method '_validate_isa' declared but not     │
-## │      │                      │ used                                                                                                           │
-## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    3 │ 36                   │ References::ProhibitDoubleSigils - Double-sigil dereference                                                    │
-## └──────┴──────────────────────┴────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
+## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
+## | Sev. | Lines                | Policy                                                                                                         |
+## |======+======================+================================================================================================================|
+## |    3 | 35                   | Subroutines::ProhibitUnusedPrivateSubroutines - Private subroutine/method '_validate_isa' declared but not     |
+## |      |                      | used                                                                                                           |
+## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
+## |    3 | 36                   | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
+## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
 __END__
