@@ -12,33 +12,16 @@ has minor  => ( is => 'ro', isa => Bool, default => 0 );
 has bugfix => ( is => 'ro', isa => Bool, default => 0 );
 
 sub run ($self) {
-    if ( $self->dist->cfg->{dist}->{cpan} && !$ENV->user_cfg->{'Pcore::API::PAUSE'}->{username} || !$ENV->user_cfg->{'Pcore::API::PAUSE'}->{password} ) {
-        say qq[You need to specify PAUSE credentials$LF];
 
-        return;
-    }
+    # check, if release can be performed
+    return if !$self->_can_release;
 
-    if ( !$self->dist->scm ) {
-        say qq[SCM is required$LF];
+    # create new version
+    my $cur_ver = $self->dist->id->{current_release} // 'v0.0.0';
 
-        return;
-    }
+    my $new_ver = $self->_compose_new_version;
 
-    my $scm = $self->dist->scm->server;
-
-    # check for uncommited changes
-    if ( $self->dist->has_uncommited_changes ) {
-        say qq[Working copy or subrepos has uncommited changes or unknown files. Release is impossible.$LF];
-
-        return;
-    }
-
-    # release is impossible, if we are not on the "default" branch
-    if ( $scm->cmd('branch')->{o}->[0] ne 'default' ) {
-        say qq[SCM should be on the "default" branch. Release is impossible.$LF];
-
-        return;
-    }
+    return if !$new_ver;
 
     # check for resolved issues without milestone
     if ( my $resolved_issues = $self->dist->build->issues->get( resolved => 1 ) ) {
@@ -51,35 +34,6 @@ sub run ($self) {
         return;
     }
 
-    # show current and new versions, take confirmation
-    my $cur_ver = $self->dist->last_release_version;
-
-    # increment version
-    my @parts = $cur_ver->{version}->@*;
-
-    if ( $self->major ) {
-        $parts[0]++;
-        $parts[1] = 0;
-        $parts[2] = 0;
-    }
-    elsif ( $self->minor ) {
-        $parts[1]++;
-        $parts[2] = 0;
-    }
-    elsif ( $self->bugfix ) {
-
-        # bugfix is impossible for v0.0.0
-        $parts[2]++ if $cur_ver ne 'v0.0.0';
-    }
-
-    my $new_ver = version->parse( 'v' . join q[.], @parts );
-
-    if ( $cur_ver eq $new_ver ) {
-        say qq[You forgot to specify release version. Release is impossible.$LF];
-
-        return;
-    }
-
     # working with issues tracker
     my $closed_issues = $self->dist->build->issues->get( closed => 1 );
 
@@ -93,6 +47,7 @@ sub run ($self) {
     }
 
     say qq[${LF}Current version is: $cur_ver];
+
     say qq[New version will be: $new_ver$LF];
 
     return if P->term->prompt( qq[Continue release process?], [qw[yes no]], enter => 1 ) ne 'yes';
@@ -104,17 +59,21 @@ sub run ($self) {
 
     say q[];
 
-    # !!!WARNING!!! start release, next changes will be hard to revert
+    # NOTE !!!WARNING!!! start release, next changes will be hard to revert
 
     # update release version in the main module
     unless ( $self->dist->module->content->$* =~ s[^(\s*package\s+\w[\w\:\']*\s+)v?[\d._]+(\s*;)][$1$new_ver$2]sm ) {
-        say q[Error updating version];
+        say q[Error updating version in the main dist module];
 
         return;
     }
 
     P->file->write_bin( $self->dist->module->path, $self->dist->module->content );
 
+    # clear cached data, important for version
+    $self->dist->clear;
+
+    # working with the issue tracker
     {
         my $cv = AE::cv;
 
@@ -184,26 +143,155 @@ sub run ($self) {
     $self->_create_changes( $new_ver, $closed_issues );
 
     # generate wiki
-    $self->dist->build->wiki->run if $self->dist->build->wiki;
+    if ( $self->dist->build->wiki ) {
+        $self->dist->build->wiki->run;
 
-    # add / remove files, possible generated by wiki command
-    $scm->cmd( 'addremove', '--subrepos' );
+        # TODO addremove and commit wiki
+    }
+
+    # addremove
+    $self->dist->scm->scm_addremove or die;
 
     # commit
-    $scm->cmd( 'commit', qq[-m"stable $new_ver"], '--subrepos' );
+    $self->dist->scm->scm_commit(qq[release $new_ver]) or die;
 
-    $scm->cmd( 'tag', '-f', 'stable', $new_ver );
+    $self->dist->scm->scm_set_tag( [ 'latest', $new_ver ], force => 1 ) or die;
 
     print 'Pushing to the upstream repository ... ';
-    $scm->cmd('push');
+
+    $self->dist->scm->scm_push or die;
+
     say 'done';
 
+    if ( $self->dist->build->docker ) {
+        require Pcore::API::DockerHub;
+
+        my $dockerhub_api = Pcore::API::DockerHub->new( { namespace => $self->dist->docker_cfg->{namespace} } );
+
+        my $dockerhub_repo = $dockerhub_api->get_repo( lc $self->dist->name );
+
+        # create dockerhub tag
+        print qq[Creating DockerHub tag "$new_ver" ... ];
+
+        $dockerhub_repo->create_build_tag( name => $new_ver, source_name => $new_ver ) or die;
+
+        say 'done';
+
+        # trigger build for new version tag
+        print qq[Trigger DockerHub build for tag "$new_ver" ... ];
+
+        $dockerhub_repo->trigger_build($new_ver) or die;
+
+        say 'done';
+
+        # trigger build for latest tag
+        print qq[Trigger DockerHub build for tag: "latest" ... ];
+
+        $dockerhub_repo->trigger_build('latest') or die;
+
+        say 'done';
+    }
+
     # upload to the CPAN if this is the CPAN distribution, prompt before upload
     $self->_upload_to_cpan if $self->dist->cfg->{dist}->{cpan};
 
     return 1;
 }
 
+sub _can_release ($self) {
+    if ( !$self->dist->scm ) {
+        say q[SCM is required];
+
+        return;
+    }
+
+    # check for uncommited changes
+    if ( !$self->dist->is_commited ) {
+        say q[Working copy or sub-repositories has uncommited changes or unknown files];
+
+        return;
+    }
+
+    # check distance from the last release
+    if ( !$self->dist->id->{current_release_distance} ) {
+        say q[No changes to release];
+
+        return;
+    }
+
+    if ( $self->dist->cfg->{dist}->{cpan} && !$ENV->user_cfg->{'Pcore::API::PAUSE'}->{username} || !$ENV->user_cfg->{'Pcore::API::PAUSE'}->{password} ) {
+        say q[You need to specify PAUSE credentials];
+
+        return;
+    }
+
+    if ( $self->dist->docker_cfg ) {
+        if ( !$ENV->user_cfg->{'Pcore::API::DockerHub'}->{api_username} || !$ENV->user_cfg->{'Pcore::API::DockerHub'}->{api_password} ) {
+            say q[You need to specify DockerHub credentials];
+
+            return;
+        }
+
+        my $dockerfile = P->file->read_bin( $self->dist->root . 'Dockerfile' );
+
+        if ( $dockerfile->$* !~ m[^FROM\s+[^:]+:v\d+[.]\d+[.]\d+$]smi ) {
+            say q[Dockerfile base image must contains version tag];
+
+            return;
+        }
+    }
+
+    return 1;
+}
+
+sub _compose_new_version ($self) {
+
+    # show current and new versions, take confirmation
+    my $cur_ver = $self->dist->id->{current_release};
+
+    if ( !$cur_ver ) {
+        if ( $self->bugfix ) {
+            say 'Bugfix has no sense on first release';
+
+            return;
+        }
+
+        $cur_ver = 'v0.0.0';
+    }
+
+    my ( $major, $minor, $bugfix ) = $cur_ver =~ /v(\d+)[.](\d+)[.](\d+)/sm;
+
+    # increment version
+    if ( $self->major ) {
+        $major++;
+        $minor  = 0;
+        $bugfix = 0;
+    }
+    elsif ( $self->minor ) {
+        $minor++;
+        $bugfix = 0;
+    }
+    elsif ( $self->bugfix ) {
+        $bugfix++;
+    }
+
+    my $new_ver = 'v' . join q[.], $major, $minor, $bugfix;
+
+    if ( $cur_ver eq $new_ver ) {
+        say q[You forgot to specify release version];
+
+        return;
+    }
+
+    if ( $new_ver ~~ $self->dist->releases ) {
+        say qq[Version $new_ver is already released];
+
+        return;
+    }
+
+    return $new_ver;
+}
+
 sub _upload_to_cpan ($self) {
     print 'Creating .tgz ... ';
 
@@ -236,7 +324,7 @@ sub _upload_to_cpan ($self) {
     else {
         say join q[ ], $res->status, $res->reason;
 
-        goto REDO if P->term->prompt( 'Retry?', [qw[yes no]], enter => 1 );
+        goto REDO if P->term->prompt( 'Retry?', [qw[yes no]], enter => 1 ) eq 'yes';
 
         say qq[Upload to CPAN failed. You should upload manually: "$tgz"];
     }
@@ -282,20 +370,20 @@ sub _create_changes ( $self, $ver, $issues ) {
 ## -----SOURCE FILTER LOG BEGIN-----
 ##
 ## PerlCritic profile "pcore-script" policy violations:
-## ┌──────┬──────────────────────┬────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
-## │ Sev. │ Lines                │ Policy                                                                                                         │
-## ╞══════╪══════════════════════╪════════════════════════════════════════════════════════════════════════════════════════════════════════════════╡
-## │    3 │ 14                   │ Subroutines::ProhibitExcessComplexity - Subroutine "run" with high complexity score (24)                       │
-## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    3 │ 98                   │ ValuesAndExpressions::ProhibitInterpolationOfLiterals - Useless interpolation of literal string                │
-## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    3 │ 266                  │ References::ProhibitDoubleSigils - Double-sigil dereference                                                    │
-## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    2 │ 44, 47, 84, 89, 110, │ ValuesAndExpressions::ProhibitLongChainsOfMethodCalls - Found method-call chain of length 4                    │
-## │      │ 126, 140             │                                                                                                                │
-## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
-## │    1 │ 262                  │ BuiltinFunctions::ProhibitReverseSortBlock - Forbid $b before $a in sort blocks                                │
-## └──────┴──────────────────────┴────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
+## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
+## | Sev. | Lines                | Policy                                                                                                         |
+## |======+======================+================================================================================================================|
+## |    3 | 14                   | Subroutines::ProhibitExcessComplexity - Subroutine "run" with high complexity score (23)                       |
+## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
+## |    3 | 53, 188              | ValuesAndExpressions::ProhibitInterpolationOfLiterals - Useless interpolation of literal string                |
+## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
+## |    3 | 354                  | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
+## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
+## |    2 | 27, 30, 38, 43, 65,  | ValuesAndExpressions::ProhibitLongChainsOfMethodCalls - Found method-call chain of length 4                    |
+## |      | 85, 99, 147          |                                                                                                                |
+## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
+## |    1 | 350                  | BuiltinFunctions::ProhibitReverseSortBlock - Forbid $b before $a in sort blocks                                |
+## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
 __END__
